/**
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Copyright (C) OX Software GmbH, 2006-2011
 */

define('oxguard/mail/keymanager', ['oxguard/core/og_http'],
    function (og_http) {
    'use strict';

    // Check the list of previously fetched key results to see if the email is there.  If so, return, otherwise query backend
    function checkRecips (email, baton) {
        var gkeys = baton.model.get('gkeys');
        if (gkeys === undefined) { // If undefined, create gkeys now
            gkeys = [];
            baton.model.set('gkeys', gkeys);
            getKeyResult (email, baton);
            return;
        }
        var result;
        gkeys.forEach(function (key) {
            if (key.email === email) {  // If email matches, and resulted, then return result
                if (key.pending === true) {
                    result = null;
                } else {
                    result = key.result;
                }
                return (false);
            }
        });
        if (result !== undefined) return (result);
        getKeyResult (email, baton);
        return (null);
    }

    // Query guard server for key result
    function getKeyResult (email, baton) {
        var result = {
            email: email,
            result: null,
            pending: true
        };
        baton.model.get('gkeys').push(result);
        var params = '&email=' + email + '&cid=' + ox.context_id + '&from=' +
        encodeURIComponent(baton.model.get('from')[1]) + '&userid=' + ox.user_id +
        '&session=' + ox.session;
        og_http.get(ox.apiRoot + '/oxguard/pgpmail?action=getkey', params)
        .done(function (data) {//Get the key
            try {
                if (data.inline) { // If needs inline, then set
                    if (baton.model.set('PGPFormat', 'inline'));
                }
                if (data.pgp === true) {
                    updateKeys (email, baton, 'pgp');
                    return;
                }
                if (data.guest === true) {
                    updateKeys (email, baton, 'guest');
                }
            } catch (e) {
                console.log(e);
            }
        })
        .fail (function () {
            updateKeys (email, baton, 'fail');
        });
    }

    // Update the list of gkeys with the result
    function updateKeys (email, baton, result) {
        var gkeys = baton.model.get('gkeys');
        gkeys.forEach(function (key) {
            if (key.email === email) {
                key.result = result;
                key.pending = false;
            }
        });
        baton.model.set('gkeys', gkeys);
        baton.model.trigger('change:gkeys');
        updateTokens(baton);
    }

    function updateTokens(baton) {
        baton.model.trigger('change:to', baton.model, baton.model.get('to'));
        baton.model.trigger('change:cc', baton.model, baton.model.get('cc'));
        baton.model.trigger('change:bcc', baton.model, baton.model.get('bcc'));
    }

    function checkAll(baton) {
        updateTokens(baton);
    }

    // Check if an email is located in gkeys
    function isPresent (email, recips) {
        for (var i = 0; i < recips.length; i++) {
            if (recips[i][1] === email) return true;
        }
        return false;
    }

    // Clean the list of gkeys, remove tokens that were deleted
    function clean(baton) {
        var to = baton.model.get('to') || [];
        var cc = baton.model.get('cc') || [];
        var bcc = baton.model.get('bcc') || [];
        var total = to.concat(cc).concat(bcc);
        var gkeys = baton.model.get('gkeys') || [];
        var newkeys = [];
        for (var i = 0; i < gkeys.length; i++) {
            if (isPresent(gkeys[i].email, total)) newkeys.push(gkeys[i]);
        }
        baton.model.set('gkeys', newkeys);
    }

    // Check if Guest present in list
    function isGuest(baton) {
        try {
            clean(baton);
            var keys = baton.model.get('gkeys');
            if (keys === undefined) keys = [];
            for (var i = 0; i < keys.length; i++) {
                if (keys[i].result === 'guest') return true;
            }
        } catch (e) {
            console.log(e);
        }

        return false;
    }

    function getGuests(baton) {
        var guests = [];
        try {
            var keys = baton.model.get('gkeys');
            if (keys === undefined) keys = [];
            for (var i = 0; i < keys.length; i++) {
                if (keys[i].result === 'guest') guests.push(keys[i].email);
            }
        } catch (e) {
            console.log(e);
        }
        return (guests);
    }

    return {
        checkRecips: checkRecips,
        checkAll: checkAll,
        clean: clean,
        isGuest: isGuest,
        getGuests: getGuests
    };

});
