define('pgp_mail/toggle-encryption', [
    'mailvelope/main',
    'io.ox/core/capabilities',
    'gettext!oxguard',
    'settings!oxguard'
], function (api, capabilities, gt, settings) {

    'use strict';

    var ToggleEncryptionView = Backbone.View.extend({
        initialize: function () {
            this.listenTo(this.model, 'change:encrypt', function (model, val) {
                this.$('i.fa').toggleClass('encrypted', val);
                //gt('Click to enable encryption')
                //gt('Click to disable encryption')
            });
        },
        className: 'toggle-encryption',
        events: {
            'click i.fa': 'toggle'
        },
        toggle: function () {
            if (this.encryption_forced) {
                require (['io.ox/core/notifications'], function (notify) {
                    notify.yell('error', gt('Reply must be encrypted'));
                });
            } else {
                this.model.set('encrypt', !this.model.get('encrypt'));
                if (this.model.get('encrypt')) {
                    try {
                        var fromArray = this.model.get('from');
                        if (window.oxguarddata.primaryEmail !== undefined) {
                            if (window.oxguarddata.primaryEmail !== fromArray[0][1]) {
                                var warn = settings.get('warnEmailDifferent');
                                if (warn === undefined || warn === false) {
                                    displayWarning();
                                }
                            }
                        }
                    } catch (e) {
                        console.log(e);
                    }

                }
            }
        },
        forceEncryption: function () {
            this.encryption_forced = true;
            this.model.set('encrypt', true);
            this.render();
        },
        render: function () {
            var guard = false;
            if (window.oxguarddata !== undefined) {
                if (window.oxguarddata.settings !== undefined) {
                    guard = (window.oxguarddata.settings.oxguard === true);
                }
            }
            if (api.isMailvelopeSupported() || guard || this.encryption_forced)
                {
                this.$el.empty().append(
                    $('<i class="fa fa-2x">')
                    .toggleClass('encrypted', !!this.model.get('encrypt'))
                );
            }
            return this;
        }
    });

    function displayWarning () {
        require (['io.ox/core/tk/dialogs'], function (dialogs) {
            var dialog = new dialogs.CreateDialog({ width: 450, height: 300, center: true, enter: 'ok' });
            dialog.header($('<h4>').text(gt('Warning')));
            var text = $('<p>').text(gt('Sending an encrypted email from an account other than %s may cause problems with the recipient being able to reply.  Consider using your primary account.', window.oxguarddata.primaryEmail));
            var warn = $('<input type="checkbox" id="warnEmail">');
            var warntext = $('<span class="selectable" style="padding-left:10px;">').text(gt('Do not warn me again'));
            warntext.click(function () {
                warn.click();
            });
            warn.click(function (e) {
                settings.set('warnEmailDifferent', $(e.target).is(':checked'));
                settings.save();
            });
            dialog.getBody().append(text).append('<hr>').append(warn).append(warntext);
            dialog
            .addPrimaryButton('ok', gt('OK'), 'ok')
            .show();
        });
    }

    return {
        View: ToggleEncryptionView
    };
});
